package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.extension.api.R;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.common.SMSCaptcha;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.vo.SportDicts;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@Api(tags = "通用接口")
@RestController
@RequestMapping("/")
public class CommController extends BaseController {

    @Autowired
    SMSCaptcha smsCaptcha;

    @GetMapping("/{client}/sms/captcha/{phone}")
    @ApiOperation(value="发送短信验证码", notes = "默认有效时长 5 分钟", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sendSMSCaptcha(@ApiParam(value = "客户端类型", allowableValues = "app,shop") @PathVariable String client,
                                       @ApiParam(value = "手机号码") @PathVariable String phone) throws Exception {
        if (StringUtils.isEmpty(phone)) {
            return ResponseBean.error("手机号码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (phone.length() != 11) {
            return ResponseBean.error("手机号码只能为11位数字", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        smsCaptcha.send(phone, smsCaptcha.randCaptcha(6));

        return ResponseBean.success("发送验证码到手机号"+phone+"成功");
    }

    @PostMapping("/{client}/sms/captcha/{phone}")
    @ApiOperation(value="校验短信验证码", notes = "校验短信验证码", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean validateSMSCaptcha(@ApiParam(value = "客户端类型", allowableValues = "app,shop") @PathVariable String client,
                                           @ApiParam(value = "手机号码") @PathVariable String phone,
                                           @ApiParam(value = "验证码") @RequestParam String captcha) throws Exception {
        if (!smsCaptcha.validate(phone, captcha)) {
            return ResponseBean.error("验证码不正确或者已过期", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        return ResponseBean.success("success");
    }
}
