package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITaMatchService;
import com.yunzhi.niucai.entity.TaMatch;

import java.util.List;

/**
 * <p>
    * 比赛  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-02
 */

@Api(tags = "比赛 ")
@RestController
@RequestMapping("/")
public class TaMatchController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaMatchController.class);

    @Autowired
    public ITaMatchService iTaMatchService;

    /**
     * 查询当前可用混合比赛
     * @param lottery
     * @return
     */
    @GetMapping("/app/sport/mix-match")
    @ApiOperation(value="查询当前可用混合比赛", notes = "查询当前可用混合比赛", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getMixedMatch(@ApiParam(value = "竞彩类型", allowableValues = "football,basketball") @RequestParam String lottery) throws Exception {
        List<TaMatch> matchList = iTaMatchService.getMixedMatch(lottery);
        return ResponseBean.success(matchList);
    }

    /**
     * 查询最新的竞彩比赛结果
     * @return
     * @throws Exception
     */
    @GetMapping("/app/sport/result/last")
    @ApiOperation(value="最新竞彩开奖结果", notes = "不区分竞彩彩种", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getLastResult() throws Exception {
        List<TaMatch> result = iTaMatchService.getLastResult();
        return ResponseBean.success(result);
    }

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/app/sport/result",method= RequestMethod.GET)
    @ApiOperation(value="竞彩开奖结果列表", notes = "竞彩开奖结果列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getResult(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                  @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                  @ApiParam("彩种类型: 可用类型, 请查阅彩种字典") @RequestParam(value ="lotteryId", required = false) String lotteryId) throws Exception {

        IPage<TaMatch> page = new Page<>(pageNum, pageSize);
        IPage<TaMatch> result = iTaMatchService.getResult(page, lotteryId);
        return ResponseBean.success(result);
    }

//
//    /**
//     * 分页查询列表
//     * @param pageNum
//     * @param pageSize
//     * @return
//     */
//    @RequestMapping(value="/taMatch",method= RequestMethod.GET)
//    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taMatchList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
//									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
//
//		    IPage<TaMatch> pg = new Page<>(pageNum, pageSize);
//            QueryWrapper<TaMatch> queryWrapper = new QueryWrapper<>();
//            queryWrapper.orderByDesc("create_date");
//
//            IPage<TaMatch> result = iTaMatchService.page(pg, queryWrapper);
//            return ResponseBean.success(result);
//    }
//
//    /**
//     * 保存对象
//     * @param taMatch 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taMatch",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taMatchAdd(@ApiParam("保存内容") @RequestBody TaMatch taMatch) throws Exception{
//
//        if (iTaMatchService.save(taMatch)){
//            return ResponseBean.success(taMatch);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taMatch/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taMatchDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaMatchService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taMatch 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taMatch/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taMatchUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaMatch taMatch) throws Exception{
//
//        if (iTaMatchService.updateById(taMatch)){
//            return ResponseBean.success(iTaMatchService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taMatch/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taMatchGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iTaMatchService.getById(id));
//    }
}
