package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.enums.StatusEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITdLotteryService;
import com.yunzhi.niucai.entity.TdLottery;

/**
 * <p>
    * 彩种 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-08-27
 */

@Api(tags = "彩种管理")
@RestController
@RequestMapping("/")
public class TdLotteryController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TdLotteryController.class);

    @Autowired
    public ITdLotteryService iTdLotteryService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/lottery",method= RequestMethod.GET)
    @ApiOperation(value="获取当前开通彩种", notes = "如果 isDigit = true 则只返回数字彩", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdLotteryList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									  @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                      @ApiParam("是否数字彩") @RequestParam(value ="isDigit", required = false) Boolean isDigit,
                                      @ApiParam(value = "客户端", allowableValues = "app, shop") @PathVariable("client") String client) throws Exception{

        IPage<TdLottery> pg = new Page<>(pageNum, pageSize);
        IPage<TdLottery> result = iTdLotteryService.getDetailPaged(pg, isDigit);

        return ResponseBean.success(result);
    }

//
//    /**
//     * 保存对象
//     * @param tdLottery 实体对象
//     * @return
//     */
//    @RequestMapping(value="/tdLottery",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean tdLotteryAdd(@ApiParam("保存内容") @RequestBody TdLottery tdLottery) throws Exception{
//
//        if (iTdLotteryService.save(tdLottery)){
//            return ResponseBean.success(tdLottery);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/tdLottery/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean tdLotteryDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTdLotteryService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param tdLottery 实体对象
//     * @return
//     */
//    @RequestMapping(value="/tdLottery/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean tdLotteryUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TdLottery tdLottery) throws Exception{
//
//        if (iTdLotteryService.updateById(tdLottery)){
//            return ResponseBean.success(iTdLotteryService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/tdLottery/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean tdLotteryGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iTdLotteryService.getById(id));
//    }
}
