package com.yunzhi.niucai.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.yunzhi.niucai.common.JWTUtils;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.common.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;


@Slf4j
@Component
public class PermissionInterceptor implements HandlerInterceptor {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {
        log.info("Request URI: {}", request.getRequestURI());

        if (request.getRequestURI().startsWith("/api/aliPay")){
            return true;
        }
        // JWT
        String jws = JWTUtils.getToken(request);
        if (StringUtils.isEmpty(jws)) {
            responseTokenError(response, "鉴权失败, 请先进行登录操作");
            return false;
        }

        try {
            JWTUtils.verify(jws);
        } catch (Exception e) {
            log.error("鉴权失败: {}", e.getMessage());
            responseTokenError(response, "鉴权失败, 请重新登录");
            return false;
        }

        // 刷新 TOKEN
        String newTk = JWTUtils.getToken(request);
        JWTUtils.refresh(newTk, response);

        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object obj, ModelAndView mv) throws Exception {
        // 刷新 TOKEN
        String jws = JWTUtils.getToken(request);
        JWTUtils.refresh(jws, response);
    }

    private void responseTokenError(HttpServletResponse response, String error) throws IOException {
        response.addHeader("Content-type", "application/json");
        response.getOutputStream().write(
                JSONObject.toJSONBytes(
                        ResponseBean.error(error, ResponseBean.ERROR_AUTH_FAIL)
                ));
    }
}
