package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.BizUtils;
import com.yunzhi.niucai.common.DateUtils;
import com.yunzhi.niucai.common.EncryptUtils;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.SportStatusEnum;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaBettingPlanItemMapper;
import com.yunzhi.niucai.mapper.TaBettingPlanMapper;
import com.yunzhi.niucai.mapper.TaBettingPlanSportDetailMapper;
import com.yunzhi.niucai.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.xml.namespace.QName;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 投注方案 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-08
 */
@Service
public class TaBettingPlanServiceImpl extends ServiceImpl<TaBettingPlanMapper, TaBettingPlan> implements ITaBettingPlanService {

    @Autowired
    ITdLotteryService iTdLotteryService;

    @Autowired
    ITdSportPassService iTdSportPassService;

    @Autowired
    ITaMatchService iTaMatchService;

    @Autowired
    ITaMatchWayService iTaMatchWayService;

    @Autowired
    ITaMatchOddsService iTaMatchOddsService;

    @Autowired
    ITaShopService iTaShopService;

    @Autowired
    ITaCustomerBettingService iTaCustomerBettingService;

    @Autowired
    ITaOrderService iTaOrderService;

    @Autowired
    ITaShopLotteryService iTaShopLotteryService;

    @Autowired
    ITaLotteryResultService iTaLotteryResultService;

    @Autowired
    TaBettingPlanItemMapper taBettingPlanItemMapper;

    @Autowired
    TaBettingPlanSportDetailMapper taBettingPlanSportDetailMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean betting(TaBettingPlan taBettingPlan, TaCustomer customer) throws Exception {
        // 各种校验
        validateBetting(taBettingPlan);

        taBettingPlan.setCustomerId(customer.getCustomerId());
        taBettingPlan.setCustomerName(customer.getName());
        taBettingPlan.setStatus(StatusEnum.READY.getCode());

        if (StringUtils.isEmpty(taBettingPlan.getAliaTitle())) {
            String aliaTitle = taBettingPlan.getIsSport()
                ? String.format("%s %s", taBettingPlan.getLotteryName(), taBettingPlan.getPassName())
                    : String.format("%s", taBettingPlan.getLotteryName());
            taBettingPlan.setAliaTitle(aliaTitle);
        }

        // 入库
        saveBettingPlan(taBettingPlan);

        // 生成我的下单
        TaCustomerBetting taCustomerBetting = iTaCustomerBettingService.saveByBetting(taBettingPlan);

        // 下单
        TaOrder order = iTaOrderService.placeOrderBy(taBettingPlan, taCustomerBetting);

        // 支付接口

        return true;
    }

    @Override
    public TaBettingPlan getDetailOfPlanById(Integer planId) throws Exception {
        TaBettingPlan taBettingPlan = getById(planId);
        if (null == taBettingPlan || StatusEnum.DELETED.getCode().equals(taBettingPlan.getStatus())) {
            return null;
        }

        List<TaBettingPlanItem> itemList = getBettingPlanItemsByPlan(taBettingPlan);
        taBettingPlan.setItemList(itemList);

        if (null != itemList && taBettingPlan.getIsSport()) {
            for (TaBettingPlanItem item: itemList) {
                List<TaBettingPlanSportDetail> detailList = getBettingPlanSprotsByItem(item);
                item.setDetailList(detailList);
            }
        }

        return taBettingPlan;
    }

    private List<TaBettingPlanSportDetail> getBettingPlanSprotsByItem(TaBettingPlanItem item) throws Exception {
        QueryWrapper<TaBettingPlanSportDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("item_no", item.getItemNo());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("item_no");
        queryWrapper.orderByAsc("serial_no");

        return taBettingPlanSportDetailMapper.selectList(queryWrapper);
    }

    private List<TaBettingPlanItem> getBettingPlanItemsByPlan(TaBettingPlan taBettingPlan) throws Exception {
        QueryWrapper<TaBettingPlanItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("plan_id", taBettingPlan.getPlanId());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("item_no");

        return taBettingPlanItemMapper.selectList(queryWrapper);
    }

    private void validateShop(TaBettingPlan taBettingPlan) throws Exception {
        if (null == taBettingPlan.getShopId()) {
            throw new Exception("请选择投注店铺");
        }

        TaShop taShop = iTaShopService.getById(taBettingPlan.getShopId());
        if (null == taShop) {
            throw new Exception("投注店铺不存在");
        }

        // 店铺状态不对
        if (!StatusEnum.NORMAL.getCode().equals(taShop.getStatus())) {
            throw new Exception("投注店铺状态异常, 请更换");
        }

        taBettingPlan.setShopName(taShop.getName());

        // 店铺彩种
        TaShopLottery taShopLottery = iTaShopLotteryService.getSettingBy(taBettingPlan.getShopId(), taBettingPlan.getLotteryId());
        if (null == taShopLottery) {
            throw new Exception("当前店铺没有相关彩种设置");
        }

        if (null == taShopLottery.getIsOpen() || !taShopLottery.getIsOpen()) {
            throw new Exception("店铺没有开售当前彩种");
        }

        // 出票额限制
        if (null != taShopLottery.getMinCash()) {
            if (taBettingPlan.getTotalPrice() < taShopLottery.getMinCash()) {
                throw new Exception("出票额不能低于" + String.format("%.2f", (float) (taShopLottery.getMinCash() / 100)) + "元");
            }
        }

        // 销售时间
        if (taBettingPlan.getIsSport()) {
            String currentTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("HH:mm"));
//            if (currentTime.compareTo(taShopLottery.getOpeningTime()) < 0 || currentTime.compareTo(taShopLottery.getClosingTime()) > 0) {
//                throw new Exception("非当前店铺彩票开售时间");
//            }
        }

        return;
    }

    private void validateBetting(TaBettingPlan taBettingPlan) throws Exception {
        if (null == taBettingPlan.getLotteryId()) {
            throw new Exception("投注彩种不能未空");
        }

        TdLottery tdLottery = iTdLotteryService.getDetailById(taBettingPlan.getLotteryId());
        if (null == tdLottery) {
            throw new Exception("投注彩种不合法");
        }
        taBettingPlan.setLotteryName(tdLottery.getName());
        taBettingPlan.setIsSport(tdLottery.getIsSport());
        taBettingPlan.setIsDigit(tdLottery.getIsDigit());

        // 校验彩种
        validateLottery(taBettingPlan, tdLottery);

        // 校验明细
        if (null == taBettingPlan.getItemList()) {
            throw new Exception("请设置投注内容");
        }

        //
        if (null == taBettingPlan.getTimes()) {
            throw new Exception("请设置投注倍数");
        }

        // 如果是体彩
        if (tdLottery.getIsSport()) {
            if (StringUtils.isEmpty(taBettingPlan.getPassCode())) {
                throw new Exception("过关方式不能为空");
            }

            TdSportPass tdSportPass = iTdSportPassService.getById(taBettingPlan.getPassCode());
            if (null == tdSportPass) {
                throw new Exception("过关方式不合法");
            }
            taBettingPlan.setPassName(tdSportPass.getName());

            List<TaBettingPlanItem> itemList = taBettingPlan.getItemList();
            if (null == itemList || tdSportPass.getUnitNum() > itemList.size()) {
                throw new Exception("投注比赛场次数量不正确");
            }

//            List<Integer> noteNumList = new ArrayList<>();
            List<List<String>> noteList = new ArrayList<>();
            for (TaBettingPlanItem item : taBettingPlan.getItemList()) {
                validateSport(taBettingPlan, item, tdLottery);
//                noteNumList.add(item.getDetailList().size());
                List<String> playList = new ArrayList<>();
                for (TaBettingPlanSportDetail sportDetail : item.getDetailList()) {
                    playList.add(sportDetail.getOdds());
                }

                noteList.add(playList);
            }

            // 校验注数
            List<List<List<String>>> mockCombList = BizUtils.mockAllCombination(noteList, tdSportPass);
            int totalNums = 0;
            for (List<List<String>> it : mockCombList) {
                if (null != it) {
                    totalNums += it.size();
                }
            }

            if (null == taBettingPlan.getNotesNum() || !taBettingPlan.getNotesNum().equals(totalNums)) {
                throw new Exception("投注总注数计算不正确, 应该为: " + totalNums);
            }

            // 总金额
            int totalPrice = totalNums * tdLottery.getUnitPrice() * taBettingPlan.getTimes();
            if (null == taBettingPlan.getTotalPrice() || !taBettingPlan.getTotalPrice().equals(totalPrice)) {
                throw new Exception("投注总金额计算不正确");
            }

            if (totalPrice > tdLottery.getMaxPrice()) {
                throw new Exception("投注总金额不能超过" + String.format("%.2f", (float) tdLottery.getMaxPrice() / 100 ) + "元");
            }
        }

        // 如果是数字彩
        if (tdLottery.getIsDigit()) {
            Integer totalCharges = 0;
            Integer totalNoteNum = 0;
            for (TaBettingPlanItem item : taBettingPlan.getItemList()) {
                validateDigit(taBettingPlan, item, tdLottery);
                totalCharges += item.getCharges();
                totalNoteNum += item.getAmount();
            }

            totalCharges *= taBettingPlan.getTimes();
            if (null == taBettingPlan.getTotalPrice()) {
                taBettingPlan.setTotalPrice(totalCharges);
            }
            if (null == taBettingPlan.getNotesNum()) {
                taBettingPlan.setNotesNum(totalNoteNum);
            }

            if (!taBettingPlan.getTotalPrice().equals(totalCharges)) {
                throw new Exception("投注总金额计算不正确");
            }

            // 不能超过限额
            Integer maxPrice = tdLottery.getMaxPrice();
            if (null != taBettingPlan.getIsAdd() && taBettingPlan.getIsAdd()) {
                maxPrice = tdLottery.getMaxAddPrice();
            }
            if (maxPrice < totalCharges) {
                throw new Exception("投注总金额不能超过" + String.format("%.2f", (float) maxPrice / 100 ) + "元");
            }

        }

        // 校验店铺, 放到最后, 因为有金额校验
        validateShop(taBettingPlan);

        return;
    }

    private void validateLottery(TaBettingPlan taBettingPlan, TdLottery tdLottery) throws Exception {
        if (!StatusEnum.NORMAL.getCode().equals(tdLottery.getStatus())) {
            throw new Exception("当前投注彩种被停用");
        }

        if (null == taBettingPlan.getTimes() || tdLottery.getMaxTimes() < taBettingPlan.getTimes()) {
            throw new Exception("当前投注倍数太大(小), 不能超过" + tdLottery.getMaxTimes());
        }

        // 投注期数
        if (null != taBettingPlan.getIsDigit() && taBettingPlan.getIsDigit()) {
            // 获取最近一期
            TaLotteryResult taLotteryResult = iTaLotteryResultService.getLastResultByLottery(tdLottery);
            if (null == taLotteryResult || StringUtils.isEmpty(taLotteryResult.getIssueNo())) {
                throw new Exception("投注彩种缺失开奖结果相关数据");
            }

            String lastOpeningDate = taLotteryResult.getOpeningDate();
            Integer lastIssueNo = Integer.valueOf(taLotteryResult.getIssueNo());

            // 默认期数是最近一次开奖 + 1
            Integer newIssueNo = lastIssueNo + 1;

            // 查询当前彩种的停售时间
            List<TdLotteryOnSale> onSaleSettings = tdLottery.getOnSaleSettings();
            if (null == onSaleSettings || onSaleSettings.size() < 1) {
                throw new Exception("投注彩种缺失开奖配置相关数据");
            }

            // 找出今天对应的开奖设置
            LocalDateTime now = LocalDateTime.now();
            int weekDay = now.getDayOfWeek().getValue();
            TdLotteryOnSale saleSetting = getSaleSetting(onSaleSettings, weekDay);

            // 如果是今天开奖
            if (weekDay == saleSetting.getWeekDay() && DateUtils.today().equals(lastOpeningDate)) {
                // 找出店铺设置的停售时间
                TaShopLottery taShopLottery = iTaShopLotteryService.getSettingBy(taBettingPlan.getShopId(), taBettingPlan.getLotteryId());
                if (null == taShopLottery) {
                    throw new Exception("当前店铺没有相关彩种设置");
                }

                // 如果超过了店铺的停售时间, 那么今天购买的只能是下期
                String currentTime = now.format(DateTimeFormatter.ofPattern("HH:mm"));
                if (currentTime.compareTo(taShopLottery.getClosingTime()) >= 0) {
                    newIssueNo += 1;
                }
            }

            taBettingPlan.setIssueNo(String.valueOf(newIssueNo));
        }
    }

    private TdLotteryOnSale getSaleSetting(List<TdLotteryOnSale> onSaleSettings, int weekDay) {
        TdLotteryOnSale onSale = null;
        for (TdLotteryOnSale item: onSaleSettings) {
            if (item.getWeekDay() >= weekDay) {
                onSale = item;
                break;
            }
        }
        if (null == onSale) {
            onSale = onSaleSettings.get(0);
        }
        return onSale;
    }

    private void validateSport(TaBettingPlan taBettingPlan,
                                       TaBettingPlanItem taBettingPlanItem,
                                       TdLottery tdLottery) throws Exception {
        // 获取比赛信息
        TaMatch taMatch = iTaMatchService.getById(taBettingPlanItem.getMatchId());
        if (null == taMatch) {
            throw new Exception("未投注比赛或者比赛不存在");
        }
        taBettingPlanItem.setLeagueId(taMatch.getLeagueId());
        taBettingPlanItem.setLeagueName(taMatch.getLeagueName());
        taBettingPlanItem.setMatchWeek(taMatch.getMatchWeek());
        taBettingPlanItem.setMatchDate(taMatch.getMatchDate());
        taBettingPlanItem.setHomeTeamId(taMatch.getHomeTeamId());
        taBettingPlanItem.setHomeTeamName(taMatch.getHomeTeamName());
        taBettingPlanItem.setAwayTeamId(taMatch.getAwayTeamId());
        taBettingPlanItem.setAwayTeamName(taMatch.getAwayTeamName());

        // 如果当前比赛不是在售
        if (!SportStatusEnum.SALE.getCode().equals(taMatch.getStatus())) {
            throw new Exception("投注比赛非在售状态");
        }

        // 投注内容
        if (null == taBettingPlanItem.getDetailList() || taBettingPlanItem.getDetailList().size() == 0) {
            throw new Exception("未找到比赛的投注内容");
        }

        // 当前比赛玩法
        Map<String, TaMatchWay> taMatchWayMap = iTaMatchWayService.getGroupedByWay(taMatch.getMatchId(), tdLottery.getLotteryId());
        if (null == taMatchWayMap) {
            throw new Exception("未找到投注比赛玩法设置");
        }

//        // 当前比赛赔率
//        Map<String, TaMatchOdds> taMatchOddsMap = iTaMatchOddsService.getGroupedByRule(taMatch.getMatchId());
//        if (null == taMatchOddsMap) {
//            throw new Exception("未找到投注比赛赔率设置");
//        }

        // 单关-过关验证
        boolean isSinglePass = iTdSportPassService.getSinglePassCode().equals(taBettingPlan.getPassCode());

        // 玩法等验证
        for (TaBettingPlanSportDetail detail: taBettingPlanItem.getDetailList()) {
            detail.setMatchId(taMatch.getMatchId());

            if (StringUtils.isEmpty(detail.getOdds())) {
                throw new Exception("未找到投注比赛明细赔率设置");
            }

            TaMatchWay taMatchWay = taMatchWayMap.get(detail.getWayCode());
            if (null == taMatchWay) {
                throw new Exception("未找到投注比赛明细玩法设置");
            }

            if (null == taMatchWay.getIsOn() || !taMatchWay.getIsOn()) {
                throw new Exception("投注比赛明细未开启购彩玩法");
            }

            if (isSinglePass) {
                if (null == taMatchWay.getIsSingle() || !taMatchWay.getIsSingle()) {
                    throw new Exception("投注比赛明细未开启单关玩法");
                }
            } else {
                if (null == taMatchWay.getIsPass() || !taMatchWay.getIsPass()) {
                    throw new Exception("投注比赛明细未开启过关玩法");
                }
            }
        }

    }

    private void validateDigit(TaBettingPlan taBettingPlan,
                               TaBettingPlanItem taBettingPlanItem,
                               TdLottery tdLottery) throws Exception {
        if (null == taBettingPlanItem.getAmount() || 0 == taBettingPlanItem.getAmount()) {
            throw new Exception("投注注数不能为空");
        }

        if (tdLottery.getFirstNums() > 0 && StringUtils.isEmpty(taBettingPlanItem.getFirstNums())) {
            throw new Exception("前区号码不能为空");
        }

        if (tdLottery.getSecondNums() > 0 && StringUtils.isEmpty(taBettingPlanItem.getSecondNums())) {
            throw new Exception("后区号码不能为空");
        }

        // 胆拖
        if (null == taBettingPlanItem.getIsDan()) {
            taBettingPlanItem.setIsDan(false);
        }

        // 复式
        if (null == taBettingPlanItem.getIsMulti()) {
            taBettingPlanItem.setIsMulti(false);
        }

        // 注数
        int amount = computeAmount(taBettingPlanItem, tdLottery);
        if (amount != taBettingPlanItem.getAmount()) {
            throw new Exception("投注注数计算不正确");
        }

        Integer unitPrice = tdLottery.getUnitPrice();
        if (null != taBettingPlan.getIsAdd() && taBettingPlan.getIsAdd()) {
            unitPrice += tdLottery.getUnitAddPrice();
        }

        Integer charges = taBettingPlanItem.getAmount() * unitPrice;
        if (null == taBettingPlanItem.getCharges() || 0 == taBettingPlanItem.getCharges()) {
            taBettingPlanItem.setCharges(charges);
        }

        if (!charges.equals(taBettingPlanItem.getCharges())) {
            throw new Exception("投注金额不正确");
        }

        return;
    }

    private void saveBettingPlan(TaBettingPlan taBettingPlan) throws Exception {
        if (!save(taBettingPlan)) {
            throw new Exception("生成投注方案出错");
        }

        for (TaBettingPlanItem item: taBettingPlan.getItemList()) {
            item.setPlanId(taBettingPlan.getPlanId());

            int effectRows = taBettingPlanItemMapper.insert(item);
            if (effectRows < 1) {
                throw new Exception("生成投注方案明细出错");
            }

            if (null != item.getDetailList() && item.getDetailList().size() > 0) {
                for(TaBettingPlanSportDetail detail: item.getDetailList()) {

                    List<TaMatchOdds> oddsList = iTaMatchOddsService.getByMatchAndRule(taBettingPlan.getLotteryId(), detail.getMatchId(), detail.getRuleCode());
                    if (null == oddsList || oddsList.size() == 0) {
                        throw new Exception("校验投注赔率出错");
                    }
                    TaMatchOdds odds = oddsList.get(0);
                    detail.setRuleName(odds.getRuleName());
                    detail.setWayCode(odds.getWayCode());
                    detail.setWayName(odds.getWayName());
                    detail.setItemNo(item.getItemNo());
                    if (taBettingPlanSportDetailMapper.insert(detail) < 1) {
                        throw new Exception("生成投注方案玩法明细出错");
                    }
                }
            }
        }
    }

    private int computeAmount(TaBettingPlanItem item, TdLottery lottery) throws Exception {
        // 排列三
        if ("p3".equals(lottery.getLotteryId())) {
            return computeP3Amount(item, lottery);
        }

        // 排列五
        if ("p5".equals(lottery.getLotteryId())) {
            return computeDirectlyAmount(item);
        }

        // 胆拖
        if (item.getIsDan()) {
            int firstNum = BizUtils.computeDanAmount(item.getFirstDan().split(",").length,
                    item.getFirstNums().split(",").length,
                    lottery.getFirstNums());

            int secondNum = 1;
            if (!StringUtils.isEmpty(item.getSecondDan())) {
                secondNum = BizUtils.computeDanAmount(item.getSecondDan().split(",").length,
                        item.getSecondNums().split(",").length,
                        lottery.getSecondNums());
            }

            return firstNum * secondNum;
        }

        // 复式
        if (item.getIsMulti()) {
            int firstNum = EncryptUtils.combination(item.getFirstNums().split(",").length, lottery.getFirstNums());

            int secondNum = 1;
            if (!StringUtils.isEmpty(item.getSecondNums())) {
                secondNum = EncryptUtils.combination(item.getSecondNums().split(",").length, lottery.getSecondNums());
            }

            return firstNum * secondNum;
        }

        // 其余情况
        return 1;
    }

    /**
     * 排列三的算法特殊
     * 拉出来单独算
     * @param item
     * @param lottery
     * @return
     */
    private int computeP3Amount(TaBettingPlanItem item, TdLottery lottery) throws Exception {
        String playWay = item.getPlayWay();
        int nums = item.getFirstNums().split(",").length;

        // 如果是组选3
        if ("组选3".equals(playWay)) {
            if (!item.getIsMulti()) {
                // 单式, 肯定就 1 注
                return 1;
            } else {
                // 复式
                if (nums < 2) {
                    throw new Exception("组选3复式 至少2个数字");
                }

                // 2 * 所有数字拿出 2 个组合的可能
                return 2 * EncryptUtils.combination(nums, 2);
            }
        }

        // 如果是组选 6
        if ("组选6".equals(playWay)) {
            return EncryptUtils.combination(nums, lottery.getFirstNums());
        }

        // 直选
        return computeDirectlyAmount(item);
    }

    /**
     * p3, p5 直选
     * 支持每一位多选情况
     * @param item
     * @return
     */
    private int computeDirectlyAmount(TaBettingPlanItem item) {
        int res = 1; // 最差是 1 注
        for (String num: item.getFirstNums().split(",")) {
            res *= num.split(" ").length;
        }
        return res;
    }
}
