package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaCustomerBettingItemMapper;
import com.yunzhi.niucai.mapper.TaCustomerBettingMapper;
import com.yunzhi.niucai.mapper.TaCustomerBettingSportMapper;
import com.yunzhi.niucai.service.ITaBettingPlanService;
import com.yunzhi.niucai.service.ITaCustomerBettingService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 客户投注  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-09
 */
@Service
public class TaCustomerBettingServiceImpl extends ServiceImpl<TaCustomerBettingMapper, TaCustomerBetting> implements ITaCustomerBettingService {

    @Autowired
    ITaBettingPlanService iTaBettingPlanService;

    @Autowired
    TaCustomerBettingItemMapper taCustomerBettingItemMapper;

    @Autowired
    TaCustomerBettingSportMapper taCustomerBettingSportMapper;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public TaCustomerBetting saveByBetting(TaBettingPlan plan) throws Exception {
        TaCustomerBetting taCustomerBetting = new TaCustomerBetting();
        taCustomerBetting.setCustomerId(plan.getCustomerId());
        taCustomerBetting.setCustomerName(plan.getCustomerName());
        taCustomerBetting.setShopId(plan.getShopId());
        taCustomerBetting.setShopName(plan.getShopName());
        taCustomerBetting.setPlanId(plan.getPlanId());
        taCustomerBetting.setPlanName(plan.getAliaTitle());
        taCustomerBetting.setLotteryId(plan.getLotteryId());
        taCustomerBetting.setLotteryName(plan.getLotteryName());
        taCustomerBetting.setIssueNo(plan.getIssueNo());
        taCustomerBetting.setBettingTimes(plan.getTimes());
        taCustomerBetting.setIsAdd(plan.getIsAdd());
        taCustomerBetting.setNotesNum(plan.getNotesNum());
        taCustomerBetting.setTotalPrice(plan.getTotalPrice());
        taCustomerBetting.setExpectedEarning(plan.getExpectedEarning());
        taCustomerBetting.setIsPrinted(false);
        taCustomerBetting.setIsOpen(false);
        taCustomerBetting.setIsWinning(false);
        taCustomerBetting.setStatus(StatusEnum.READY.getCode());

        if (!save(taCustomerBetting)) {
            throw new Exception("生成我的投注失败");
        }

        if (null != plan.getItemList()) {
            for (TaBettingPlanItem item: plan.getItemList()) {
                TaCustomerBettingItem bettingItem = new TaCustomerBettingItem();
                bettingItem.setBettingId(taCustomerBetting.getBettingId());
                bettingItem.setPlanItemNo(item.getItemNo());
                bettingItem.setStatus(StatusEnum.NORMAL.getCode());

                if (taCustomerBettingItemMapper.insert(bettingItem) < 1) {
                    throw new Exception("生成我的投注明细失败");
                }

                if (null != item.getDetailList()) {
                    for (TaBettingPlanSportDetail detail: item.getDetailList()) {
                        TaCustomerBettingSport bettingSport = new TaCustomerBettingSport();
                        bettingSport.setBettingId(taCustomerBetting.getBettingId());
                        bettingSport.setItemNo(bettingItem.getItemNo());
                        bettingSport.setDetailNo(detail.getSerialNo());
                        bettingSport.setMatchId(item.getMatchId());
                        bettingSport.setRuleCode(detail.getRuleCode());
                        bettingSport.setOddsOrdered(detail.getOdds());
                        bettingSport.setStatus(StatusEnum.NORMAL.getCode());

                        if (taCustomerBettingSportMapper.insert(bettingSport) < 1) {
                            throw new Exception("生成我的投注明细失败");
                        }
                    }
                }
            }
        }

        return taCustomerBetting;
    }

    @Override
    public Map<String, Object> getSummaryOf(Integer id) throws Exception {
        TaCustomerBetting betting = getById(id);
        if (null == betting || StatusEnum.DELETED.getCode().equals(betting.getStatus())) {
            throw new Exception("数据不存在, 投注ID不正确");
        }

        // 投注明细
        betting.setItemList(getBettingItemByBetting(betting));

        // 竞彩明细
        betting.setBettingSports(getBettingSportByBetting(betting));

        // 查询方案
        TaBettingPlan taBettingPlan = iTaBettingPlanService.getDetailOfPlanById(betting.getPlanId());

        Map<String, Object> result = new HashMap<>();
        result.put("betting", betting);
        result.put("plan", taBettingPlan);

        return result;
    }

    @Override
    public List<TaCustomerBetting> getAllUnOpenBettingsBy(TdLottery lottery, TaLotteryResult lotteryResult) throws Exception {
        QueryWrapper<TaCustomerBetting> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("lottery_id", lottery.getLotteryId());
        queryWrapper.eq("issue_no", lotteryResult.getIssueNo());
        queryWrapper.eq("is_open", false);
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());

        List<TaCustomerBetting> list = list(queryWrapper);
        if (null != list && list.size() > 0) {
            for (TaCustomerBetting betting : list) {
                if (lottery.getIsSport()) {
                    betting.setBettingSports(getBettingSportByBetting(betting));
                }
                betting.setItemList(getBettingItemByBetting(betting));
                betting.setBettingPlan(iTaBettingPlanService.getDetailOfPlanById(betting.getPlanId()));
            }
        }

        return list;
    }

    private List<TaCustomerBettingSport> getBettingSportByBetting(TaCustomerBetting betting) throws Exception {
        QueryWrapper<TaCustomerBettingSport> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("betting_id", betting.getBettingId());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("serial_no");

        return taCustomerBettingSportMapper.selectList(queryWrapper);
    }

    private List<TaCustomerBettingItem> getBettingItemByBetting(TaCustomerBetting betting) throws Exception {
        QueryWrapper<TaCustomerBettingItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("betting_id", betting.getBettingId());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByAsc("serial_no");

        return taCustomerBettingItemMapper.selectList(queryWrapper);
    }
}
