package com.yunzhi.niucai.service.impl;

import com.yunzhi.niucai.common.DateUtils;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.TaBettingPlan;
import com.yunzhi.niucai.entity.TaCustomerBetting;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.enums.OrderTypeEnum;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaOrderMapper;
import com.yunzhi.niucai.service.ITaOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 订单主表  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-09
 */
@Service
public class TaOrderServiceImpl extends ServiceImpl<TaOrderMapper, TaOrder> implements ITaOrderService {

    @Autowired
    TaOrderMapper taOrderMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public TaOrder placeOrderBy(TaBettingPlan taBettingPlan, TaCustomerBetting taCustomerBetting) throws Exception {
        TaOrder order = new TaOrder();
        order.setOrderId(nextOrderNo("BL", 16, true));
        order.setOrderType(OrderTypeEnum.BUY_LOTTERY.getCode());
        order.setAmount(taCustomerBetting.getBettingTimes());
        order.setCharge(taCustomerBetting.getTotalPrice());
        order.setShopId(taCustomerBetting.getShopId());
        order.setCustomerId(taCustomerBetting.getCustomerId());
        order.setBettingId(taCustomerBetting.getBettingId());
        order.setIsPayed(false);    // 未支付
        order.setStatus(StatusEnum.NORMAL.getCode());

        if (!save(order)) {
            throw new Exception("生成购彩订单失败");
        }

        return order;
    }

    private String nextOrderNo(String prefix, int length, boolean padDate) throws Exception {
        List<String> parts = new ArrayList<>();
        parts.add(prefix);

        String dtStr = null;
        if (padDate) {
            dtStr = DateUtils.todayCompact();
            parts.add(dtStr);
        }

        int fillLength = length - prefix.length();
        if (padDate) {
            fillLength -= dtStr.length();
        }

        String nextNo = taOrderMapper.nextval(String.join("-", parts)).toString();

        String zeros = StringUtils.repeat("0", fillLength - nextNo.length());
        parts.add(zeros+nextNo);

        return String.join("-", parts);
    }
}
