package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.entity.TaCustomer;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaShopMapper;
import com.yunzhi.niucai.service.ITaShopService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 店铺 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-08-28
 */
@Service
public class TaShopServiceImpl extends ServiceImpl<TaShopMapper, TaShop> implements ITaShopService {

    @Autowired
    TaShopMapper taShopMapper;

    @Override
    public Map<String, Object> getAvailableList(TaCustomer customer, String lotteryId, String location) throws Exception {
        // 获取推荐我的店铺
        TaShop ofMine = taShopMapper.getRecommendOf(customer.getCustomerId(), lotteryId);

        // 获取系统推荐列表
        List<TaShop> recommList = taShopMapper.getSysRecommendList(lotteryId);
        // 不包含推荐我的店铺
        if (null != ofMine && null !=recommList) {
            for(TaShop x : recommList) {
                if (x.getShopId().equals(ofMine.getShopId())) {
                    recommList.remove(x);
                    break;
                }
            }
        }

        // 获取最近距离的 10 家店铺
        // 不包含上面的结果
        List<Integer> notIn = new ArrayList<>();
        if (null != ofMine) {
            notIn.add(ofMine.getShopId());
        }
        if (null != recommList) {
            for (TaShop s : recommList) {
                notIn.add(s.getShopId());
            }
        }
        String[] loc = location.split(",");
        List<TaShop> shopList = taShopMapper.getNearestShops(lotteryId, loc[0], loc[1], notIn, 10);

        if (null == ofMine && null == recommList && null == shopList) {
            return null;
        }

        Map<String, Object> result = new HashMap<>();
        result.put("mine", ofMine);
        result.put("recommend", recommList);
        result.put("nearest", shopList);

        return result;
    }

    @Override
    public TaShop getByInviteCode(String inviteCode) throws Exception {
        QueryWrapper<TaShop> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("invite_code", inviteCode);
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());

        return getOne(queryWrapper);
    }
}
